from typing import Generic, TypeVar

from rainbowadn.core import *
from rainbowadn.inlining import *
from .keyed import *

__all__ = ('KeyValue',)

KVKeyType = TypeVar('KVKeyType')
KVValueType = TypeVar('KVValueType')


class KeyValue(Keyed[KVKeyType], IAuto, Generic[KVKeyType, KVValueType]):
    value: HashPoint[KVValueType]

    def __init__(self, *args):
        IAuto.__init__(self, *args)
        key, self.value = self.hashpoints()
        Keyed.__init__(self, key)

    @classmethod
    async def off(
            cls, key: HashPoint[KVKeyType], value: HashPoint[KVValueType]
    ) -> 'KeyValue[KVKeyType, KVValueType]':
        assert isinstance(key, HashPoint)
        assert isinstance(value, HashPoint)
        return await cls.auto_off(key, value)

    @classmethod
    def of(cls, vk: KVKeyType, vv: KVValueType) -> 'KeyValue[KVKeyType, KVValueType]':
        assert isinstance(vk, Mentionable)
        assert isinstance(vv, Mentionable)
        return cls.auto_of(vk, vv)

    @classmethod
    def f(
            cls, f0: RainbowFactory[KVKeyType], f1: RainbowFactory[KVValueType]
    ) -> RainbowFactory['KeyValue[KVKeyType, KVValueType]']:
        assert isinstance(f0, RainbowFactory)
        assert isinstance(f1, RainbowFactory)
        return cls.auto_f(f0, f1)
