from typing import Generic, Optional, TypeVar

from rainbowadn.core import *
from .binarycreation import *
from .binaryprotocolized import *
from .binarysplit import *

__all__ = ('ProtocolizedBinarySplit',)

TreeType = TypeVar('TreeType')
ActiveKeyType = TypeVar('ActiveKeyType')
MetaDataType = TypeVar('MetaDataType')


class ProtocolizedBinarySplit(
    Generic[ActiveKeyType, MetaDataType, TreeType]
):
    def __init__(
            self,
            protocol: BinaryCreation[ActiveKeyType, MetaDataType, TreeType],
            split: BinarySplit[ActiveKeyType, MetaDataType, TreeType],
            tree: TreeType
    ):
        assert isinstance(protocol, BinaryCreation)
        assert isinstance(split, BinarySplit)
        self.protocol = protocol
        self.split = split
        self.tree = tree

    def protocolize(self, tree: TreeType) -> BinaryProtocolized[ActiveKeyType, MetaDataType, TreeType]:
        return BinaryProtocolized(self.protocol, tree)

    def protocolized(self) -> BinaryProtocolized[ActiveKeyType, MetaDataType, TreeType]:
        return self.protocolize(self.tree)

    def protocolizedl(self) -> BinaryProtocolized[ActiveKeyType, MetaDataType, TreeType]:
        return self.protocolize(self.split.treel)

    def protocolizedr(self) -> BinaryProtocolized[ActiveKeyType, MetaDataType, TreeType]:
        return self.protocolize(self.split.treer)

    @classmethod
    async def split_of(
            cls,
            protocolized: BinaryProtocolized[ActiveKeyType, MetaDataType, TreeType]
    ) -> Optional[
        'ProtocolizedBinarySplit[ActiveKeyType, MetaDataType, TreeType]'
    ]:
        assert isinstance(protocolized, BinaryProtocolized)
        if (split := await protocolized.split()) is None:
            return None
        else:
            return ProtocolizedBinarySplit(protocolized.creation, split, protocolized.tree)

    async def verify_metadata(self) -> bool:
        assert_true(
            await self.protocol.verify_metadata(
                self.split.treel, self.split.treer, self.split.key, self.split.metadata
            )
        )
        return True
